#! /bin/sh
#
#   GNUstep.sh.in
#
#   Shell initialization for the GNUstep environment.
#
#   Copyright (C) 1997 Free Software Foundation, Inc.
#
#   Author:  Scott Christley <scottc@net-community.com>
#
#   This file is part of the GNUstep Makefile Package.
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#   
#   You should have received a copy of the GNU General Public
#   License along with this library; see the file COPYING.LIB.
#   If not, write to the Free Software Foundation,
#   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

#
# Set the GNUstep system root and local root
#
GNUSTEP_ROOT=/usr/GNUstep
GNUSTEP_SYSTEM_ROOT=/usr/GNUstep/System
GNUSTEP_FLATTENED=
if [ -z "$LIBRARY_COMBO" ]; then
  LIBRARY_COMBO=gnu-gnu-gnu
fi
export GNUSTEP_ROOT GNUSTEP_SYSTEM_ROOT GNUSTEP_FLATTENED LIBRARY_COMBO

GNUSTEP_MAKEFILES=$GNUSTEP_SYSTEM_ROOT/Makefiles
export GNUSTEP_MAKEFILES

GNUSTEP_LOCAL_ROOT=/usr/GNUstep/Local
GNUSTEP_NETWORK_ROOT=/usr/GNUstep/Network
GNUSTEP_USER_ROOT=

if [ -z "$GNUSTEP_USER_ROOT" ]; then
  if [ ! -z "$WINDIR" ]; then
    GNUSTEP_USER_ROOT=`echo $HOMEDRIVE/$HOMEPATH/GNUstep | tr '\\\' '/'`
  else
    GNUSTEP_USER_ROOT=~/GNUstep
  fi
fi

ac_save_ifs="$IFS"; IFS=":"

CYGPATH=
ac_word=cygpath.exe
ac_dummy="$PATH"
for ac_dir in $ac_dummy; do
  test -z "$ac_dir" && ac_dir=.
  if test -f $ac_dir/$ac_word; then
    CYGPATH=$ac_word
    break
  fi
done

IFS="$ac_save_ifs"

if [ -z "$CYGPATH" ]; then
  CYGPATH=$GNUSTEP_MAKEFILES/cygpath.sh
fi

# test to see if GNUSTEP_ROOT is in Win32 standard form and flag it
if [ -z "`echo $GNUSTEP_SYSTEM_ROOT | sed 's|^[a-zA-Z]:/.*$||'`" ]; then
  CONVERT_WIN_PATHS=yes
fi

export GNUSTEP_LOCAL_ROOT GNUSTEP_NETWORK_ROOT GNUSTEP_USER_ROOT


#
# Determine the host information
#
if [ -z "$GNUSTEP_HOST" ]; then
  # Not all OSs (e.g. FreeBSD < 4.0) have pushd/popd
  tmpdir=`pwd`; cd /tmp
  GNUSTEP_HOST=`$GNUSTEP_MAKEFILES/config.guess`
  GNUSTEP_HOST=`$GNUSTEP_MAKEFILES/config.sub $GNUSTEP_HOST`
  cd $tmpdir
fi

if [ -z "$GNUSTEP_HOST_CPU" ]; then
  GNUSTEP_HOST_CPU=`$GNUSTEP_MAKEFILES/cpu.sh $GNUSTEP_HOST`
  GNUSTEP_HOST_CPU=`$GNUSTEP_MAKEFILES/clean_cpu.sh $GNUSTEP_HOST_CPU`
fi

if [ -z "$GNUSTEP_HOST_VENDOR" ]; then
  GNUSTEP_HOST_VENDOR=`$GNUSTEP_MAKEFILES/vendor.sh $GNUSTEP_HOST`
  GNUSTEP_HOST_VENDOR=`$GNUSTEP_MAKEFILES/clean_vendor.sh $GNUSTEP_HOST_VENDOR`
fi

if [ -z "$GNUSTEP_HOST_OS" ]; then 
  GNUSTEP_HOST_OS=`$GNUSTEP_MAKEFILES/os.sh $GNUSTEP_HOST`
  GNUSTEP_HOST_OS=`$GNUSTEP_MAKEFILES/clean_os.sh $GNUSTEP_HOST_OS`
fi

export GNUSTEP_HOST GNUSTEP_HOST_CPU GNUSTEP_HOST_VENDOR GNUSTEP_HOST_OS

#
# Add the GNUstep tools directories to the path
#
if [ -z "$GNUSTEP_PATHPREFIX_LIST" ]; then
  if [ "$CONVERT_WIN_PATHS" = "yes" ]; then
    G_U_R=`$CYGPATH -u "$GNUSTEP_USER_ROOT"`
    G_L_R=`$CYGPATH -u "$GNUSTEP_LOCAL_ROOT"`
    G_N_R=`$CYGPATH -u "$GNUSTEP_NETWORK_ROOT"`
    G_S_R=`$CYGPATH -u "$GNUSTEP_SYSTEM_ROOT"`
    GNUSTEP_PATHPREFIX_LIST=$G_U_R:$G_L_R:$G_N_R:$G_S_R
  else
    GNUSTEP_PATHPREFIX_LIST=$GNUSTEP_USER_ROOT:$GNUSTEP_LOCAL_ROOT:$GNUSTEP_NETWORK_ROOT:$GNUSTEP_SYSTEM_ROOT
  fi
  export GNUSTEP_PATHPREFIX_LIST
fi

tmp_IFS="$IFS"
IFS=:
temp_path=
for dir in $GNUSTEP_PATHPREFIX_LIST; do
  temp_path=$temp_path$dir/Tools:
  if [ -z "$GNUSTEP_FLATTENED" ]; then
    temp_path=$temp_path$dir/Tools/${GNUSTEP_HOST_CPU}/${GNUSTEP_HOST_OS}/${LIBRARY_COMBO}:
    temp_path=$temp_path$dir/Tools/${GNUSTEP_HOST_CPU}/${GNUSTEP_HOST_OS}:
  fi
done
IFS="$tmp_IFS"
if [ -z "$PATH" ]; then
  PATH="$temp_path"
else
  if ( echo ${PATH}| grep -v "${temp_path}" >/dev/null ); then
    PATH="${temp_path}${PATH}"
  fi
fi
export PATH

# directory for gcc's 3.x shared libraries
GCC_SO_DIR=NONE
export GCC_SO_DIR
. $GNUSTEP_MAKEFILES/ld_lib_path.sh

if [ "$CONVERT_WIN_PATHS" = "yes" ]; then
  gnustep_class_path="$G_U_R/Libraries/Java:$G_L_R/Libraries/Java:$G_N_R/Libraries/Java:$G_S_R/Libraries/Java"
else
  gnustep_class_path="$GNUSTEP_USER_ROOT/Libraries/Java:$GNUSTEP_LOCAL_ROOT/Libraries/Java:$GNUSTEP_NETWORK_ROOT/Libraries/Java:$GNUSTEP_SYSTEM_ROOT/Libraries/Java"
fi

if [ -z "$CLASSPATH" ]; then
  CLASSPATH="$gnustep_class_path"
else
  if ( echo ${CLASSPATH}| grep -v "${gnustep_class_path}" >/dev/null ); then
    CLASSPATH="$CLASSPATH:$gnustep_class_path"
  fi
fi

export CLASSPATH

#
# Perform any user initialization
#
if [ -f ~/GNUstep/GNUstep.sh ]
then
  . ~/GNUstep/GNUstep.sh
fi

#
# Run 'make_services' in background if possible - updates cache of services
# and applications/file-extensions known to the applications.
#
if [ -z "$GNUSTEP_FLATTENED" ]; then
  TDIR=$GNUSTEP_SYSTEM_ROOT/Tools/$GNUSTEP_HOST_CPU/$GNUSTEP_HOST_OS
  if [ -f $TDIR/$LIBRARY_COMBO/make_services ]
  then
    $TDIR/$LIBRARY_COMBO/make_services &
  else
    if [ -f $TDIR/make_services ]
    then
      $TDIR/make_services &
    fi
  fi
else
  TDIR=$GNUSTEP_SYSTEM_ROOT/Tools
  if [ -f $TDIR/make_services ]
  then
    $TDIR/make_services &
  fi
fi

